/*
	SuperCollider real time audio synthesis system
    Copyright (c) 2002 James McCartney. All rights reserved.
	http://www.audiosynth.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
*/


#include "SC_PlugIn.h"

static InterfaceTable *ft;

struct ___PROJECTNAMEASIDENTIFIER___ : public Unit
{
	// persistent property of this UGen. "m" prefix denotes "member'.
	uint32 mDelayLength;
	uint32 mPosition;
	float *mData;
};


//////////////////////////////////////////////////////////////////////////////////////////////////


extern "C"
{
	void load(InterfaceTable *inTable);

	void ___PROJECTNAMEASIDENTIFIER____next_notfull(___PROJECTNAMEASIDENTIFIER___ *unit, int inNumSamples);
	void ___PROJECTNAMEASIDENTIFIER____next_full(___PROJECTNAMEASIDENTIFIER___ *unit, int inNumSamples);
	void ___PROJECTNAMEASIDENTIFIER____Ctor(___PROJECTNAMEASIDENTIFIER___ *unit);
	void ___PROJECTNAMEASIDENTIFIER____Dtor(___PROJECTNAMEASIDENTIFIER___ *unit);	
}


//////////////////////////////////////////////////////////////////////////////////////////////////


// calculation function when the buffer has not yet been filled
void ___PROJECTNAMEASIDENTIFIER____next_notfull(___PROJECTNAMEASIDENTIFIER___ *unit, int inNumSamples)
{
	// get the pointers to the input and output buffers
	float *out = OUT(0);
	float *in = IN(0);
	
	float *data = unit->mData;	
	uint32 length = unit->mDelayLength;
	uint32 position = unit->mPosition;
	
	bool wrapped = false;

	// perform a loop for the number of samples in the control period.
	for (int i=0; i < inNumSamples; ++i)
	{
		// get old value in delay line		
		float z = data[position];
		
		// store new value in delay line
		data[position] = in[i];
		
		// see if the position went to the end of the buffer 
		
		if (++position >= length)
		{			
			position = 0;		// go back to beginning			
			wrapped = true;		// indicate we have wrapped.
			
			// change the calculation function
			// next time, the ___PROJECTNAMEASIDENTIFIER____next_full function will be called
			SETCALC(___PROJECTNAMEASIDENTIFIER____next_full);
			
		}
		
		// if we have not yet wrapped, then z is garbage from the uninitialized 
		// buffer, so output zero. If we have wrapped, then z is a good value.
		out[i] = wrapped ? z : 0.f;
	}

	// store the position back to the struct
	unit->mPosition = position;	
}


//////////////////////////////////////////////////////////////////////////////////////////////////


// calculation function when the buffer has been filled
void ___PROJECTNAMEASIDENTIFIER____next_full(___PROJECTNAMEASIDENTIFIER___ *unit, int inNumSamples)
{
	// get the pointers to the input and output buffers
	float *out = OUT(0);
	float *in = IN(0);
	
	float *data = unit->mData;	
	uint32 length = unit->mDelayLength;
	uint32 position = unit->mPosition;
	
	// perform a loop for the number of samples in the control period.
	for (int i=0; i < inNumSamples; ++i)		
	{
		// get old value in delay line
		float z = data[position];
		
		// store new value in delay line
		data[position] = in[i];
		
		// see if the position went to the end of the buffer 
		if (++position >= length)
			position = 0;
		
		out[i] = z;
	}
	
	// store the position back to the struct
	unit->mPosition = position;	
}


////////////////////////////////////////////////////////////////////////////////////////////////////////


void ___PROJECTNAMEASIDENTIFIER____Ctor(___PROJECTNAMEASIDENTIFIER___* unit)

{
	SETCALC(___PROJECTNAMEASIDENTIFIER____next_notfull);
	
	// set the delay length according to the (fixed) time parameter
	unit->mDelayLength = (uint32) (IN0(1) * SAMPLERATE);
	unit->mPosition = 0;
	
	// allocate the buffer
	// RTAlloc allocates out of the finite real time memory pool of the server.	
	unit->mData = (float*)RTAlloc(unit->mWorld, unit->mDelayLength * sizeof(float));
	
	___PROJECTNAMEASIDENTIFIER____next_notfull(unit, 1);	
}


//////////////////////////////////////////////////////////////////////////////////////////////////


void ___PROJECTNAMEASIDENTIFIER____Dtor(___PROJECTNAMEASIDENTIFIER___* unit)
{
	// free the buffer
	RTFree(unit->mWorld, unit->mData);	
}


//////////////////////////////////////////////////////////////////////////////////////////////////


void load(InterfaceTable *inTable)
{
	ft = inTable;

	DefineDtorUnit(___PROJECTNAMEASIDENTIFIER___);
}


////////////////////////////////////////////////////////////////////////////////////////////////////////
